"""

Parameters W2D:
    Cells:           Simple cell with just bias and gain from Worm2D
    Chem Synapses:   Continuous transmission of output of one cell to next
    Gap junctions:   Electrical connection; current linearly depends on difference in voltages

ASSESSMENT:
    Useful as tested in a number of 2D worm body models & very few parameters

"""

from neuroml import PulseGenerator

from neuroml import GapJunction

from c302.bioparameters import c302ModelPrototype


class ParameterisedModel(c302ModelPrototype):
    def __init__(self):
        super(ParameterisedModel, self).__init__()
        self.level = "W2D"
        self.custom_component_types_definitions = [
            "cell_W2D.xml",
            "custom_synapses.xml",
        ]

        self.set_default_bioparameters()
        self.print_("Set default parameters for %s" % self.level)

    def set_default_bioparameters(self):
        self.add_bioparameter("initial_memb_pot", "-45 mV", "BlindGuess", "0.1")

        self.add_bioparameter(
            "neuron_to_neuron_elec_syn_gbase", "1 nS", "BlindGuess", "0.1"
        )
        self.add_bioparameter(
            "neuron_to_muscle_elec_syn_gbase", "1 nS", "BlindGuess", "0.1"
        )

        self.add_bioparameter(
            "unphysiological_offset_current", "0 pA", "KnownError", "0"
        )  # Can be activated later
        self.add_bioparameter(
            "unphysiological_offset_current_del", "0 ms", "KnownError", "0"
        )
        self.add_bioparameter(
            "unphysiological_offset_current_dur", "2000 ms", "KnownError", "0"
        )

    def create_models(self):
        self.create_generic_muscle_cell()
        self.create_generic_neuron_cell()
        self.create_offsetcurrent()
        self.create_neuron_to_neuron_syn()
        self.create_neuron_to_muscle_syn()

    def create_generic_muscle_cell(self):
        self.generic_muscle_cell = CellW2D(id="GenericMuscleCell")

    def create_generic_neuron_cell(self):
        self.generic_neuron_cell = CellW2D(id="GenericNeuronCell")

    def create_offsetcurrent(self):
        self.offset_current = PulseGenerator(
            id="offset_current",
            delay=self.get_bioparameter("unphysiological_offset_current_del").value,
            duration=self.get_bioparameter("unphysiological_offset_current_dur").value,
            amplitude=self.get_bioparameter("unphysiological_offset_current").value,
        )

    def create_neuron_to_neuron_syn(self):
        self.neuron_to_neuron_exc_syn = OutputSynapse(id="neuron_to_neuron_w2d")

        self.neuron_to_neuron_elec_syn = GapJunction(
            id="neuron_to_neuron_elec_syn",
            conductance=self.get_bioparameter("neuron_to_neuron_elec_syn_gbase").value,
        )

    def create_neuron_to_muscle_syn(self):
        self.neuron_to_muscle_exc_syn = OutputSynapse(id="neuron_to_muscle_w2d")

        self.neuron_to_muscle_elec_syn = GapJunction(
            id="neuron_to_muscle_elec_syn",
            conductance=self.get_bioparameter("neuron_to_muscle_elec_syn_gbase").value,
        )

    def get_elec_syn(self, pre_cell, post_cell, type):
        if type == "neuron_to_neuron":
            gbase = self.get_bioparameter("neuron_to_neuron_elec_syn_gbase").value
            conn_id = "neuron_to_neuron_elec_syn"

        elif type == "neuron_to_muscle":
            gbase = self.get_bioparameter("neuron_to_muscle_elec_syn_gbase").value
            conn_id = "neuron_to_muscle_elec_syn"

        return GapJunction(id=conn_id, conductance=gbase)

    def get_exc_syn(self, pre_cell, post_cell, type):
        return self.neuron_to_neuron_exc_syn

    def get_inh_syn(self, pre_cell, post_cell, type):
        return self.neuron_to_neuron_exc_syn


from c302.bioparameters import NonNeuroMLCustomType


class CellW2D(NonNeuroMLCustomType):
    def __init__(self, id):
        self.id = id


class OutputSynapse(NonNeuroMLCustomType):
    def __init__(self, id):
        self.id = id



"""

Parameters C1:
    Cells:           Single compartment, conductance based cell models with HH like ion channels
    Chem Synapses:   Analogue/graded synapses; continuous transmission (voltage dependent)
    Gap junctions:   Electrical connection; current linerly depends on difference in voltages

ASSESSMENT:
    A good prospect, but cell model could be simpler. See C0

"""

from neuroml import GradedSynapse
from neuroml import GapJunction

from c302.parameters_C import ParameterisedModel as ParameterisedModel_C


class ParameterisedModel(ParameterisedModel_C):
    def __init__(self):
        super(ParameterisedModel, self).__init__()
        self.level = "C1"
        self.custom_component_types_definitions = "cell_C.xml"

        self.set_default_bioparameters()
        self.print_("Set default parameters for %s" % self.level)

    def set_default_bioparameters(self):
        param_C = ParameterisedModel_C()
        param_C.set_default_bioparameters()
        for b in param_C.bioparameters:
            if "syn" not in b.name:
                self.add_bioparameter_obj(b)

        self.add_bioparameter(
            "neuron_to_neuron_exc_syn_conductance", "0.09 nS", "BlindGuess", "0.1"
        )
        self.add_bioparameter(
            "neuron_to_muscle_exc_syn_conductance", "0.09 nS", "BlindGuess", "0.1"
        )

        self.add_bioparameter("exc_syn_delta", "5 mV", "BlindGuess", "0.1")
        self.add_bioparameter("exc_syn_vth", "0 mV", "BlindGuess", "0.1")
        self.add_bioparameter("exc_syn_erev", "0 mV", "BlindGuess", "0.1")
        self.add_bioparameter("exc_syn_k", "0.025per_ms", "BlindGuess", "0.1")

        self.add_bioparameter(
            "neuron_to_neuron_inh_syn_conductance", "0.09 nS", "BlindGuess", "0.1"
        )
        self.add_bioparameter(
            "neuron_to_muscle_inh_syn_conductance", "0.09 nS", "BlindGuess", "0.1"
        )

        self.add_bioparameter("inh_syn_delta", "5 mV", "BlindGuess", "0.1")
        self.add_bioparameter("inh_syn_vth", "0 mV", "BlindGuess", "0.1")
        self.add_bioparameter("inh_syn_erev", "-70 mV", "BlindGuess", "0.1")
        self.add_bioparameter("inh_syn_k", "0.025per_ms", "BlindGuess", "0.1")

        self.add_bioparameter(
            "neuron_to_neuron_elec_syn_gbase", "0.00052 nS", "BlindGuess", "0.1"
        )
        self.add_bioparameter(
            "neuron_to_muscle_elec_syn_gbase", "0.00052 nS", "BlindGuess", "0.1"
        )

    def create_neuron_to_neuron_syn(self):
        self.neuron_to_neuron_exc_syn = GradedSynapse(
            id="neuron_to_neuron_exc_syn",
            conductance=self.get_bioparameter(
                "neuron_to_neuron_exc_syn_conductance"
            ).value,
            delta=self.get_bioparameter("exc_syn_delta").value,
            Vth=self.get_bioparameter("exc_syn_vth").value,
            erev=self.get_bioparameter("exc_syn_erev").value,
            k=self.get_bioparameter("exc_syn_k").value,
        )

        self.neuron_to_neuron_inh_syn = GradedSynapse(
            id="neuron_to_neuron_inh_syn",
            conductance=self.get_bioparameter(
                "neuron_to_neuron_inh_syn_conductance"
            ).value,
            delta=self.get_bioparameter("inh_syn_delta").value,
            Vth=self.get_bioparameter("inh_syn_vth").value,
            erev=self.get_bioparameter("inh_syn_erev").value,
            k=self.get_bioparameter("inh_syn_k").value,
        )

        self.neuron_to_neuron_elec_syn = GapJunction(
            id="neuron_to_neuron_elec_syn",
            conductance=self.get_bioparameter("neuron_to_neuron_elec_syn_gbase").value,
        )

    def create_neuron_to_muscle_syn(self):
        self.neuron_to_muscle_exc_syn = GradedSynapse(
            id="neuron_to_muscle_exc_syn",
            conductance=self.get_bioparameter(
                "neuron_to_muscle_exc_syn_conductance"
            ).value,
            delta=self.get_bioparameter("exc_syn_delta").value,
            Vth=self.get_bioparameter("exc_syn_vth").value,
            erev=self.get_bioparameter("exc_syn_erev").value,
            k=self.get_bioparameter("exc_syn_k").value,
        )

        self.neuron_to_muscle_inh_syn = GradedSynapse(
            id="neuron_to_muscle_inh_syn",
            conductance=self.get_bioparameter(
                "neuron_to_muscle_inh_syn_conductance"
            ).value,
            delta=self.get_bioparameter("inh_syn_delta").value,
            Vth=self.get_bioparameter("inh_syn_vth").value,
            erev=self.get_bioparameter("inh_syn_erev").value,
            k=self.get_bioparameter("inh_syn_k").value,
        )

        self.neuron_to_muscle_elec_syn = GapJunction(
            id="neuron_to_muscle_elec_syn",
            conductance=self.get_bioparameter("neuron_to_muscle_elec_syn_gbase").value,
        )

    def get_elec_syn(self, pre_cell, post_cell, type):
        self.found_specific_param = False
        if type == "neuron_to_neuron":
            gbase = self.get_conn_param(
                pre_cell,
                post_cell,
                "%s_to_%s_elec_syn_%s",
                "neuron_to_neuron_elec_syn_%s",
                "gbase",
            )
            conn_id = "neuron_to_neuron_elec_syn"
        elif type == "neuron_to_muscle":
            gbase = self.get_conn_param(
                pre_cell,
                post_cell,
                "%s_to_%s_elec_syn_%s",
                "neuron_to_muscle_elec_syn_%s",
                "gbase",
            )
            conn_id = "neuron_to_muscle_elec_syn"

        if self.found_specific_param:
            conn_id = "%s_to_%s_elec_syn" % (pre_cell, post_cell)

        return GapJunction(id=conn_id, conductance=gbase)

    def get_exc_syn(self, pre_cell, post_cell, type):
        self.found_specific_param = False

        specific_param_template = "%s_to_%s_exc_syn_%s"
        if type == "neuron_to_neuron":
            conductance = self.get_conn_param(
                pre_cell,
                post_cell,
                specific_param_template,
                "neuron_to_neuron_exc_syn_%s",
                "conductance",
            )
            erev = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "erev"
            )
            delta = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "delta"
            )
            vth = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "vth"
            )
            k = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "k"
            )

            conn_id = "neuron_to_neuron_exc_syn"

        elif type == "neuron_to_muscle":
            conductance = self.get_conn_param(
                pre_cell,
                post_cell,
                specific_param_template,
                "neuron_to_muscle_exc_syn_%s",
                "conductance",
            )
            erev = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "erev"
            )
            delta = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "delta"
            )
            vth = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "vth"
            )
            k = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "exc_syn_%s", "k"
            )

            conn_id = "neuron_to_muscle_exc_syn"

        if self.found_specific_param:
            conn_id = "%s_to_%s_exc_syn" % (pre_cell, post_cell)

        return GradedSynapse(
            id=conn_id, conductance=conductance, delta=delta, Vth=vth, erev=erev, k=k
        )

    def get_inh_syn(self, pre_cell, post_cell, type):
        self.found_specific_param = False

        specific_param_template = "%s_to_%s_inh_syn_%s"
        if type == "neuron_to_neuron":
            conductance = self.get_conn_param(
                pre_cell,
                post_cell,
                specific_param_template,
                "neuron_to_neuron_inh_syn_%s",
                "conductance",
            )
            erev = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "erev"
            )
            delta = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "delta"
            )
            vth = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "vth"
            )
            k = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "k"
            )

            conn_id = "neuron_to_neuron_inh_syn"

        elif type == "neuron_to_muscle":
            conductance = self.get_conn_param(
                pre_cell,
                post_cell,
                specific_param_template,
                "neuron_to_muscle_inh_syn_%s",
                "conductance",
            )
            erev = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "erev"
            )
            delta = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "delta"
            )
            vth = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "vth"
            )
            k = self.get_conn_param(
                pre_cell, post_cell, specific_param_template, "inh_syn_%s", "k"
            )

            conn_id = "neuron_to_muscle_inh_syn"

        if self.found_specific_param:
            conn_id = "%s_to_%s_inh_syn" % (pre_cell, post_cell)

        return GradedSynapse(
            id=conn_id, conductance=conductance, delta=delta, Vth=vth, erev=erev, k=k
        )



import neuroml.loaders as loaders

import sys


nml2_files = []

if len(sys.argv) >= 2:
    for i in range(1, len(sys.argv)):
        nml2_files.append(sys.argv[i])

print("")
for nml2_file in nml2_files:
    nml2_doc = loaders.NeuroMLLoader.load(nml2_file)

    print("-------- File: %s ----------" % nml2_file)

    for cell in nml2_doc.cells:
        print("  Cell: %s; %i segments" % (cell.id, len(cell.morphology.segments)))

    for exp_two_synapse in nml2_doc.exp_two_synapses:
        print("  Synapse: %s" % (exp_two_synapse.id))

    for gap_junction in nml2_doc.gap_junctions:
        print("  Electrical Synapse: %s" % (gap_junction.id))

    for network in nml2_doc.networks:
        print("  Network: %s" % (network.id))
        pops = []
        for pop in network.populations:
            pops.append(pop.id)

        print("    Populations (%i): %s" % (len(pops), pops))

        projs = []
        connections = 0
        for proj in network.projections:
            projs.append(proj.id)
            connections += len(proj.connections)

        print(
            "    Projections:            %i with %i connections in total"
            % (len(projs), connections)
        )

        elec_projs = []
        elec_connections = 0
        for elec_proj in network.electrical_projections:
            elec_projs.append(elec_proj.id)
            elec_connections += len(elec_proj.electrical_connections)

        print(
            "    Electrical Projections: %i with %i connections in total"
            % (len(elec_projs), elec_connections)
        )

    print("")



from c302 import generate, add_new_input, print_

import neuroml.writers as writers

import importlib
import sys


if __name__ == "__main__":
    parameter_set = sys.argv[1] if len(sys.argv) == 2 else "A"

    ParameterisedModel = getattr(
        importlib.import_module("c302.parameters_%s" % parameter_set),
        "ParameterisedModel",
    )
    params = ParameterisedModel()

    cells = ["URYDL", "SMDDR", "ADAL", "RIML", "IL2VL", "RIPL"]
    cells_to_stimulate = []

    reference = "c302_%s_MultiSyns" % parameter_set

    target_directory = "examples"

    nml_doc = generate(
        reference,
        params,
        cells=cells,
        cells_to_stimulate=cells_to_stimulate,
        duration=1000,
        dt=0.1,
        target_directory=target_directory,
        verbose=True,
    )

    stim_amplitude = "0.35nA"
    add_new_input(nml_doc, "URYDL", "100ms", "200ms", stim_amplitude, params)
    add_new_input(nml_doc, "ADAL", "400ms", "200ms", stim_amplitude, params)
    add_new_input(nml_doc, "IL2VL", "700ms", "200ms", stim_amplitude, params)

    nml_file = target_directory + "/" + reference + ".net.nml"
    writers.NeuroMLWriter.write(
        nml_doc, nml_file
    )  # Write over network file written above...

    print_("(Re)written network file to: " + nml_file)



import errno
import sys
import os
import time
import importlib as IM
from collections import OrderedDict

from pyneuroml import pynml

import c302
from c302 import c302_utils

save_fig_dir = "summary/"
save_image_dir = "images"
save_image_full_dir = "%s/%s" % (save_fig_dir, save_image_dir)


def run_c302(
    config,
    parameter_set,
    prefix,
    duration,
    dt,
    simulator,
    save=False,
    show_plot_already=True,
    data_reader=c302.DEFAULT_DATA_READER,
    verbose=False,
    plot_ca=True,
    plot_connectivity=False,
    param_overrides={},
    config_param_overrides={},
    config_package="",
    target_directory="examples",
    save_fig_to=None,
):
    if save_fig_to:
        global save_fig_dir
        save_fig_dir = save_fig_to

    c302.print_(
        "********************\n\n   Going to generate c302_%s_%s and run for %sms (dt: %sms) on %s\n\n********************"
        % (parameter_set, config, duration, dt, simulator)
    )

    if not config_package:
        config_package = "c302"

    setup = IM.import_module("%s.c302_%s" % (config_package, config)).setup

    try:
        os.makedirs(target_directory)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

    cells, cells_to_stimulate, params, muscles, nml_doc = setup(
        parameter_set,
        data_reader=data_reader,
        generate=True,
        duration=duration,
        dt=dt,
        target_directory=target_directory,
        verbose=verbose,
        param_overrides=param_overrides,
        config_param_overrides=config_param_overrides,
    )

    orig_dir = os.getcwd()

    os.chdir(target_directory)

    try:
        os.makedirs(save_fig_dir)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

    lems_file = "LEMS_c302_%s_%s.xml" % (parameter_set, config)

    if simulator == "jNeuroML":
        results = pynml.run_lems_with_jneuroml(
            lems_file, nogui=True, load_saved_data=True, verbose=verbose
        )
    elif simulator == "jNeuroML_NEURON":
        results = pynml.run_lems_with_jneuroml_neuron(
            lems_file, nogui=True, load_saved_data=True, verbose=verbose
        )

    c302.print_("Finished simulation of %s and have reloaded results" % lems_file)

    c302_utils.plot_c302_results(
        results,
        config,
        parameter_set,
        directory=save_image_full_dir,
        save=save,
        show_plot_already=show_plot_already,
        data_reader=data_reader,
        plot_ca=plot_ca,
    )

    if plot_connectivity:
        c302_utils.generate_conn_matrix(nml_doc, save_fig_dir=save_image_full_dir)

    os.chdir(orig_dir)

    return cells, cells_to_stimulate, params, muscles


if __name__ == "__main__":
    if "-full" in sys.argv:
        run_c302("Full", "C", "", 300, 0.05, "jNeuroML_NEURON")

    elif "-fullA" in sys.argv:
        run_c302("Full", "A", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-fullB" in sys.argv:
        run_c302("Full", "B", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-fullC0" in sys.argv:
        run_c302("Full", "C0", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-fullC" in sys.argv:
        run_c302("Full", "C", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-fullC1" in sys.argv:
        run_c302("Full", "C1", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-fullD" in sys.argv:
        run_c302("Full", "D", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-fullD1" in sys.argv:
        run_c302("Full", "D1", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-fw" in sys.argv:
        run_c302(
            "AVB_FW",
            "C2",
            "",
            4000,
            0.05,
            "jNeuroML_NEURON",
            config_package="notebooks.configs.AVB",
            data_reader=c302.FW_DATA_READER,
        )

    elif "-fw1" in sys.argv:
        # run_c302('FW','C2','',1000,0.05,'jNeuroML_NEURON', data_reader='UpdatedSpreadsheetDataReader2', save=True)
        run_c302(
            "FW",
            "C2",
            "",
            5000,
            0.005,
            "jNeuroML_NEURON",
            data_reader=c302.FW_DATA_READER,
            save=True,
            show_plot_already=False,
        )

    elif "-fwTest" in sys.argv:
        run_c302(
            "FW",
            "C2",
            "",
            5000,
            0.05,
            "jNeuroML_NEURON",
            data_reader=c302.FW_DATA_READER,
            save=True,
            show_plot_already=True,
        )

    elif "-fullC2" in sys.argv:
        run_c302("Full", "C2", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-muscle1" in sys.argv:
        run_c302(
            "MuscleTest",
            "C2",
            "",
            2000,
            0.05,
            "jNeuroML_NEURON",
            data_reader="UpdatedSpreadsheetDataReader",
            save=True,
        )

    elif "-head1" in sys.argv:
        run_c302(
            "HeadMuscleTest",
            "C2",
            "",
            4000,
            0.05,
            "jNeuroML_NEURON",
            config_package="notebooks.configs.Head",
            data_reader="UpdatedSpreadsheetDataReader",
        )

    elif "-muscle" in sys.argv or "-muscles" in sys.argv:
        run_c302("Muscles", "C", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-musclesA" in sys.argv or "-muscA" in sys.argv:
        run_c302("Muscles", "A", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-musclesB" in sys.argv or "-muscB" in sys.argv:
        run_c302("Muscles", "B", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-musclesC0" in sys.argv or "-muscC0" in sys.argv:
        run_c302("Muscles", "C0", "", 1000, 0.05, "jNeuroML_NEURON")

    elif "-musclesC" in sys.argv or "-muscC" in sys.argv:
        run_c302("Muscles", "C", "", 2000, 0.05, "jNeuroML_NEURON")

    elif "-musclesC1" in sys.argv or "-muscC1" in sys.argv:
        run_c302("Muscles", "C1", "", 2000, 0.05, "jNeuroML_NEURON")

    elif "-musclesD" in sys.argv or "-muscD" in sys.argv:
        run_c302("Muscles", "D", "", 2000, 0.05, "jNeuroML_NEURON")

    elif "-musclesD1" in sys.argv or "-muscD1" in sys.argv:
        run_c302("Muscles", "D1", "", 2000, 0.05, "jNeuroML_NEURON")

    elif "-pharA" in sys.argv or "-pharyngealA" in sys.argv:
        run_c302("Pharyngeal", "A", "", 500, 0.01, "jNeuroML_NEURON")

    elif "-pharB" in sys.argv or "-pharyngealB" in sys.argv:
        run_c302("Pharyngeal", "B", "", 500, 0.01, "jNeuroML_NEURON")

    elif "-pharC0" in sys.argv or "-pharyngealC0" in sys.argv:
        run_c302("Pharyngeal", "C0", "", 500, 0.01, "jNeuroML_NEURON")

    elif "-pharC1" in sys.argv or "-pharyngealC1" in sys.argv:
        run_c302("Pharyngeal", "C1", "", 500, 0.01, "jNeuroML_NEURON")

    elif "-pharD" in sys.argv or "-pharyngealD" in sys.argv:
        run_c302("Pharyngeal", "D", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-pharD1" in sys.argv or "-pharyngealD1" in sys.argv:
        run_c302("Pharyngeal", "D1", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-phar" in sys.argv or "-pharyngeal" in sys.argv:
        run_c302("Pharyngeal", "C", "", 500, 0.01, "jNeuroML_NEURON")

    elif "-synsA" in sys.argv:
        run_c302("Syns", "A", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-synsB" in sys.argv:
        run_c302("Syns", "B", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-synsC0" in sys.argv:
        run_c302("Syns", "C0", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-synsC" in sys.argv:
        run_c302("Syns", "C", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-synsC1" in sys.argv:
        run_c302("Syns", "C1", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-synsD" in sys.argv:
        run_c302("Syns", "D", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-synsD1" in sys.argv:
        run_c302("Syns", "D1", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-synsW2D" in sys.argv:
        run_c302("Syns", "W2D", "", 500, 0.05, "jNeuroML_NEURON")

    elif "-socialA" in sys.argv:
        run_c302("Social", "A", "", 2500, 0.05, "jNeuroML_NEURON")

    elif "-socialB" in sys.argv:
        run_c302("Social", "B", "", 2500, 0.05, "jNeuroML_NEURON")

    elif "-socialC0" in sys.argv:
        run_c302("Social", "C0", "", 2500, 0.05, "jNeuroML_NEURON")

    elif "-socialC" in sys.argv:
        run_c302("Social", "C", "", 2500, 0.05, "jNeuroML_NEURON")

    elif "-socialC1" in sys.argv:
        run_c302("Social", "C1", "", 2500, 0.05, "jNeuroML_NEURON")

    elif "-oscA" in sys.argv:
        run_c302("Oscillator", "A", "", 600, 0.05, "jNeuroML_NEURON")

    elif "-oscB" in sys.argv:
        run_c302("Oscillator", "B", "", 600, 0.05, "jNeuroML_NEURON")

    elif "-oscC0" in sys.argv:
        run_c302("Oscillator", "C0", "", 2000, 0.025, "jNeuroML_NEURON")

    elif "-oscmC0" in sys.argv:
        run_c302("OscillatorM", "C0", "", 2000, 0.025, "jNeuroML_NEURON")

    elif "-test" in sys.argv:
        run_c302(
            "Oscillator", "C0", "", 2000, 0.025, "jNeuroML", show_plot_already=False
        )

    elif "-oscC" in sys.argv:
        run_c302("Oscillator", "C", "", 2000, 0.05, "jNeuroML_NEURON")

    elif "-oscC1" in sys.argv:
        run_c302("Oscillator", "C1", "", 2000, 0.05, "jNeuroML_NEURON")

    elif "-KatoC1" in sys.argv:
        run_c302("Kato", "C1", "", 1080, 0.05, "jNeuroML_NEURON")

    elif "-targetC0" in sys.argv:
        run_c302("TargetMuscle", "C0", "", 300, 0.05, "jNeuroML")

    elif "-twC2" in sys.argv:
        run_c302(
            "TapWithdrawal",
            "C2",
            "",
            500,
            0.05,
            "jNeuroML_NEURON",
            data_reader="UpdatedSpreadsheetDataReader",
        )

    elif "-sinusC2" in sys.argv:
        run_c302(
            "SinusoidalInputTest",
            "C2",
            "",
            1000,
            0.05,
            "jNeuroML_NEURON",
            data_reader="UpdatedSpreadsheetDataReader",
        )

    elif "-iA" in sys.argv:
        run_c302("IClamp", "A", "", 6000, 0.05, "jNeuroML")
    elif "-iB" in sys.argv:
        run_c302("IClamp", "B", "", 6000, 0.05, "jNeuroML")
    elif "-iC0" in sys.argv:
        run_c302("IClamp", "C0", "", 6000, 0.05, "jNeuroML")
    elif "-iC" in sys.argv:
        run_c302("IClamp", "C", "", 6000, 0.05, "jNeuroML")
    elif "-iC1" in sys.argv:
        run_c302("IClamp", "C1", "", 6000, 0.05, "jNeuroML", save=True)
    elif "-iD" in sys.argv:
        run_c302("IClamp", "D", "", 6000, 0.05, "jNeuroML_NEURON", save=True)
    elif "-iD1" in sys.argv:
        run_c302("IClamp", "D1", "", 6000, 0.05, "jNeuroML_NEURON", save=True)

    elif "-iw2d" in sys.argv:
        run_c302("IClamp", "W2D", "", 6000, 0.05, "jNeuroML", save=True)

    elif "-all" in sys.argv:
        print("Generating all plots")
        html = "<table>\n"

        levels = ["A", "B", "C0", "C", "C1", "C2", "D", "D1"]
        # levels = ['D','D1']
        # levels = ['C2']
        # levels = ['C0']

        durations = OrderedDict(
            [
                ("IClamp", 6000),
                ("Syns", 500),
                ("Pharyngeal", 500),
                ("Social", 2500),
                ("Oscillator", 1000),
                ("Muscles", 1000),
                ("Full", 1000),
            ]
        )

        """durations = OrderedDict([('IClamp',1000),
                                ('Syns',500),
                                ('Oscillator',1000),
                                ('OscillatorM',1000)])

        durations = OrderedDict([('IClamp',1000),
                                ('Syns',500),
                                ('Social',2500)])"""

        html += "<tr>\n"
        html += "  <td>&nbsp;</td>\n"
        for p in durations.keys():
            html += (
                '  <td align="center"><b><a href="https://github.com/openworm/c302/blob/master/c302/c302_%s.py">%s</a></b></td>\n'
                % (p, p)
            )

        html += "</tr>\n"
        for c in levels:
            print("Generating for: %s" % c)

            img_loc = save_image_dir

            html += "\n<tr>\n"
            html += (
                '  <td><b><a href="https://github.com/openworm/c302/blob/master/c302/parameters_%s.py">Params %s</a></b></td>\n'
                % (c, c)
            )
            for p in durations.keys():
                print("Params: %s" % p)
                html2 = "<h2>c302 simulation</h2>\n"
                html2 += (
                    "<p>Parameter set: <b>%s</b>; configuration: <b>%s</b></p>\n"
                    % (c, p)
                )
                html2 += "<p>Generated on: %s with c302 v%s</p><br/>\n" % (
                    time.strftime("%a, %d %b %Y", time.gmtime()),
                    c302.__version__,
                )
                html2 += "<table>\n"
                html += '  <td><a href="summary_%s_%s.html">\n' % (c, p)
                html += (
                    '    <img alt="?" src="%s/neurons_%s_%s.png" height="80"/></a>\n'
                    % (img_loc, c, p)
                )

                html += (
                    '    <br/><a href="https://github.com/openworm/c302/blob/master/examples/c302_%s_%s.net.nml">NML</a>\n'
                    % (c, p)
                )
                html += (
                    '    &nbsp;<a href="http://opensourcebrain.org/projects/celegans?explorer=https://raw.githubusercontent.com/openworm/c302/master/examples/c302_%s_%s.net.nml">OSB</a>\n'
                    % (c, p)
                )

                pres = ["neurons_", "neuron_activity_", "muscles_", "muscle_activity_"]
                for pre in pres:
                    html2 += (
                        '\n<tr>\n  <td><a href="%s/%s%s_%s.png"><img alt=" " src="%s/%s%s_%s.png" height="320"/></a></td>\n'
                        % (img_loc, pre, c, p, img_loc, pre, c, p)
                    )
                    if pre == "neurons_":
                        pre = "neuron_"
                    if pre == "muscle_activity_":
                        pre = "muscles_activity_"
                    html2 += (
                        '  <td><a href="%s/traces_%s%s_%s.png"><img alt=" " src="%s/traces_%s%s_%s.png" height="320"/></a></td>\n</tr>\n'
                        % (img_loc, pre, p, c, img_loc, pre, p, c)
                    )

                html2 += "</table>\n"
                html2 += "<table>\n"

                html2 += (
                    '\n<tr><td><a href="%s/c302_%s_%s_exc_to_neurons.png"><img alt=" " src="%s/c302_%s_%s_exc_to_neurons.png" height="320"/></a></td>\n'
                    % (img_loc, c, p, img_loc, c, p)
                )
                html2 += (
                    '\n  <td><a href="%s/c302_%s_%s_inh_to_neurons.png"><img alt=" " src="%s/c302_%s_%s_inh_to_neurons.png" height="320"/></a></td>\n'
                    % (img_loc, c, p, img_loc, c, p)
                )
                html2 += (
                    '\n  <td><a href="%s/c302_%s_%s_elec_neurons_neurons.png"><img alt=" " src="%s/c302_%s_%s_elec_neurons_neurons.png" height="320"/></a></td></tr>\n'
                    % (img_loc, c, p, img_loc, c, p)
                )

                html2 += (
                    '\n<tr><td><a href="%s/c302_%s_%s_exc_to_muscles.png"><img alt=" " src="%s/c302_%s_%s_exc_to_muscles.png" height="320"/></a></td>\n'
                    % (img_loc, c, p, img_loc, c, p)
                )
                html2 += (
                    '\n  <td><a href="%s/c302_%s_%s_inh_to_muscles.png"><img alt=" " src="%s/c302_%s_%s_inh_to_muscles.png" height="320"/></a></td>'
                    % (img_loc, c, p, img_loc, c, p)
                )
                if os.path.isfile(
                    "%s/%s/%s/c302_%s_%s_elec_neurons_muscles.png"
                    % ("examples", save_fig_dir, img_loc, c, p)
                ):
                    html2 += (
                        '\n\n  <td><a href="%s/c302_%s_%s_elec_neurons_muscles.png"><img alt=" " src="%s/c302_%s_%s_elec_neurons_muscles.png" height="320"/></a></td>'
                        % (img_loc, c, p, img_loc, c, p)
                    )
                html2 += "</tr>\n"

                if os.path.isfile(
                    "%s/%s/%s/c302_%s_%s_elec_muscles_muscles.png"
                    % ("examples", save_fig_dir, img_loc, c, p)
                ):
                    html2 += (
                        '\n<tr><td><a href="%s/c302_%s_%s_elec_muscles_muscles.png"><img alt=" " src="%s/c302_%s_%s_elec_muscles_muscles.png" height="320"/></a></td></tr>\n'
                        % (img_loc, c, p, img_loc, c, p)
                    )

                html2 += "</table>\n"

                with open(
                    "examples/%s/summary_%s_%s.html" % (save_fig_dir, c, p), "w"
                ) as f2:
                    f2.write("<html><body>%s</body></html>" % html2)
                with open(
                    "examples/%s/summary_%s_%s.md" % (save_fig_dir, c, p), "w"
                ) as f3:
                    f3.write("### Parameter config summary \n%s" % html2)

                dt = 0.05 if p != "Pharyngeal" else 0.01

                run_c302(
                    p,
                    c,
                    "",
                    durations[p],
                    dt,
                    "jNeuroML_NEURON",
                    save=True,
                    show_plot_already=False,
                    plot_connectivity=True,
                    data_reader=c302.DEFAULT_DATA_READER,
                )

                html += "  </td>\n"

            html += "</tr>\n"

        html += "</table>\n"

        with open("examples/" + save_fig_dir + "info.html", "w") as f:
            f.write("<html><body>\n%s\n</body></html>" % html)
        with open("examples/" + save_fig_dir + "README.md", "w") as f2:
            f2.write("### c302 activity summary \n%s" % (html.replace(".html", ".md")))

    else:
        run_c302("Syns", "C", "", 500, 0.05, "jNeuroML")



import sys
import os
import importlib

sys.path.insert(0, os.path.abspath("."))

import c302

import neuroml.writers as writers

range_incl = lambda start, end: range(start, end + 1)


def setup(
    parameter_set,
    generate=False,
    duration=2000,
    dt=0.05,
    target_directory="examples",
    data_reader=c302.FW_DATA_READER,
    param_overrides={},
    verbose=True,
    config_param_overrides={},
):
    ParameterisedModel = getattr(
        importlib.import_module("c302.parameters_%s" % parameter_set),
        "ParameterisedModel",
    )
    params = ParameterisedModel()

    params.set_bioparameter(
        "unphysiological_offset_current", "0pA", "Testing TapWithdrawal", "0"
    )
    params.set_bioparameter(
        "unphysiological_offset_current_del", "0 ms", "Testing TapWithdrawal", "0"
    )
    params.set_bioparameter(
        "unphysiological_offset_current_dur", "2000 ms", "Testing TapWithdrawal", "0"
    )

    """
    VA_motors = ["VA%s" % c for c in range_incl(1, 12)]
    DA_motors = ["DA%s" % c for c in range_incl(1, 9)]
    AS_motors = ["AS%s" % c for c in range_incl(1, 11)]"""
    VB_motors = ["VB%s" % c for c in range_incl(1, 11)]
    DB_motors = ["DB%s" % c for c in range_incl(1, 7)]
    DD_motors = ["DD%s" % c for c in range_incl(1, 6)]
    VD_motors = ["VD%s" % c for c in range_incl(1, 13)]

    cells = list(["AVBL", "AVBR"] + DB_motors + VD_motors + VB_motors + DD_motors)

    muscles_to_include = True

    cells_to_stimulate = []

    cells_to_plot = list(cells)
    reference = "c302_%s_FW" % parameter_set

    conns_to_include = []
    conns_to_exclude = [
        "VB2-VB4_GJ",
        "VB4-VB2_GJ",
    ]
    conn_polarity_override = {
        r"^DB\d+-DD\d+$": "inh",
        r"^VB\d+-VD\d+$": "inh",
    }
    conn_number_override = {
        "^.+-.+$": 1,
    }

    input_list = []

    """dur = '250ms'
    amp = '3pA'
    for muscle_num in range(24):
        mdlx = 'MDL0%s' % (muscle_num + 1)
        mdrx = 'MDR0%s' % (muscle_num + 1)
        #mvlx = 'MVL0%s' % (muscle_num + 1)
        #mvrx = 'MVR0%s' % (muscle_num + 1)
        
        if muscle_num >= 9:
            mdlx = 'MDL%s' % (muscle_num + 1)
            mdrx = 'MDR%s' % (muscle_num + 1)
            #mvlx = 'MVL%s' % (muscle_num + 1)
            #mvrx = 'MVR%s' % (muscle_num + 1)

        
        startd = '%sms' % (muscle_num * 10)
        #startv = '%sms' % ((stim_num * 800 + 400) + muscle_num * 30)
        
        input_list.append((mdlx, startd, dur, amp))
        input_list.append((mdrx, startd, dur, amp))"""

    input_list.append(("MVR10", "0ms", "150ms", "1pA"))
    input_list.append(("MVR11", "0ms", "150ms", "2pA"))
    input_list.append(("MVR12", "0ms", "150ms", "3pA"))
    input_list.append(("MVR13", "0ms", "150ms", "3pA"))
    input_list.append(("MVR14", "0ms", "150ms", "2pA"))
    input_list.append(("MVR15", "0ms", "150ms", "1pA"))

    input_list.append(("MVL10", "0ms", "150ms", "1pA"))
    input_list.append(("MVL11", "0ms", "150ms", "2pA"))
    input_list.append(("MVL12", "0ms", "150ms", "3pA"))
    input_list.append(("MVL13", "0ms", "150ms", "3pA"))
    input_list.append(("MVL14", "0ms", "150ms", "2pA"))
    input_list.append(("MVL15", "0ms", "150ms", "1pA"))

    input_list.append(("MDL21", "0ms", "250ms", "3pA"))
    input_list.append(("MDL22", "0ms", "250ms", "3pA"))
    input_list.append(("MDR21", "0ms", "250ms", "3pA"))
    input_list.append(("MDR22", "0ms", "250ms", "3pA"))

    amp = "4pA"
    dur = "250ms"

    for stim_num in range(15):
        for muscle_num in range(7):
            mdlx = "MDL0%s" % (muscle_num + 1)
            mdrx = "MDR0%s" % (muscle_num + 1)
            mvlx = "MVL0%s" % (muscle_num + 1)
            mvrx = "MVR0%s" % (muscle_num + 1)

            if muscle_num >= 9:
                mdlx = "MDL%s" % (muscle_num + 1)
                mdrx = "MDR%s" % (muscle_num + 1)
                mvlx = "MVL%s" % (muscle_num + 1)
                mvrx = "MVR%s" % (muscle_num + 1)

            startd = "%sms" % (stim_num * 800 + muscle_num * 30)
            startv = "%sms" % ((stim_num * 800 + 400) + muscle_num * 30)

            input_list.append((mdlx, startd, dur, amp))
            input_list.append((mdrx, startd, dur, amp))
            if muscle_num != 6:
                input_list.append((mvlx, startv, dur, amp))
                input_list.append((mvrx, startv, dur, amp))

    d_v_delay = 400

    start = 190
    motor_dur = "250ms"

    input_list.append(("AVBL", "0ms", "1e9ms", "15pA"))
    input_list.append(("AVBR", "0ms", "1e9ms", "15pA"))
    input_list.append(("DB1", "%sms" % (start), motor_dur, "3pA"))
    input_list.append(("VB1", "%sms" % (start + d_v_delay), motor_dur, "3pA"))

    i = start + 2 * d_v_delay
    j = start + 3 * d_v_delay
    for pulse_num in range(1, 15):
        input_list.append(("DB1", "%sms" % i, motor_dur, "3pA"))
        input_list.append(("VB1", "%sms" % j, motor_dur, "3pA"))
        i += d_v_delay * 2
        j += d_v_delay * 2

    # input_list = []
    # input_list.append(('AVBL', '0ms', '1900ms', '15pA'))
    # input_list.append(('AVBR', '0ms', '1900ms', '15pA'))

    config_param_overrides["input"] = input_list

    param_overrides = {
        "mirrored_elec_conn_params": {
            r"^AVB._to_DB\d+\_GJ$_elec_syn_gbase": "0.001 nS",
            r"^AVB._to_VB\d+\_GJ$_elec_syn_gbase": "0.001 nS",
            r"^DB\d+_to_DB\d+\_GJ$_elec_syn_gbase": "0.001 nS",
            #'^DB\d+_to_DB\d+\_GJ$_elec_syn_p_gbase': '0.08 nS',
            #'^DB\d+_to_DB\d+\_GJ$_elec_syn_sigma': '0.2 per_mV',
            #'^DB\d+_to_DB\d+\_GJ$_elec_syn_mu': '-20 mV',
            r"^VB\d+_to_VB\d+\_GJ$_elec_syn_gbase": "0.001 nS",
            #'^VB\d+_to_VB\d+\_GJ$_elec_syn_p_gbase': '0.1 nS',
            #'^VB\d+_to_VB\d+\_GJ$_elec_syn_sigma': '0.3 per_mV',
            #'^VB\d+_to_VB\d+\_GJ$_elec_syn_mu': '-30 mV',
            #'VB2_to_VB4_elec_syn_gbase': '0 nS',
            r"^DB\d+_to_VB\d+\_GJ$_elec_syn_gbase": "0 nS",
            r"^DB\d+_to_DD\d+\_GJ$_elec_syn_gbase": "0 nS",
            r"^VB\d+_to_VD\d+\_GJ$_elec_syn_gbase": "0 nS",
            #'^VD\d+_to_DD\d+\_GJ$_elec_syn_gbase': '0 nS',
            "DD1_to_MVL08_elec_syn_gbase": "0 nS",
            "VD2_to_MDL09_elec_syn_gbase": "0 nS",
        },
        r"^VB\d+_to_VB\d+$_exc_syn_conductance": "18 nS",
        r"^VB\d+_to_VB\d+$_exc_syn_ar": "0.19 per_s",
        r"^VB\d+_to_VB\d+$_exc_syn_ad": "73 per_s",
        r"^VB\d+_to_VB\d+$_exc_syn_beta": "2.81 per_mV",
        r"^VB\d+_to_VB\d+$_exc_syn_vth": "-22 mV",
        r"^VB\d+_to_VB\d+$_exc_syn_erev": "10 mV",
        r"^DB\d+_to_DB\d+$_exc_syn_conductance": "20 nS",
        r"^DB\d+_to_DB\d+$_exc_syn_ar": "0.08 per_s",
        r"^DB\d+_to_DB\d+$_exc_syn_ad": "18 per_s",
        r"^DB\d+_to_DB\d+$_exc_syn_beta": "0.21 per_mV",
        r"^DB\d+_to_DB\d+$_exc_syn_vth": "-10 mV",
        r"^DB\d+_to_DB\d+$_exc_syn_erev": "10 mV",
        "initial_memb_pot": "-50 mV",
        "AVBR_to_DB4_exc_syn_conductance": "0 nS",
        #'VB4_to_VB5_exc_syn_conductance': '0 nS',
        "AVBL_to_VB2_exc_syn_conductance": "0 nS",
        "AVBR_to_VD3_exc_syn_conductance": "0 nS",
        #'^DB\d+_to_DD\d+$_exc_syn_conductance': '0 nS',
        #'^DD\d+_to_DB\d+$_inh_syn_conductance': '0 nS',
        #'^VB\d+_to_VD\d+$_exc_syn_conductance': '0 nS',
        #'^VD\d+_to_VB\d+$_inh_syn_conductance': '0 nS',
        "DD1_to_VB2_inh_syn_conductance": "0 nS",
        "neuron_to_muscle_exc_syn_conductance": "0.5 nS",
        r"^DB\d+_to_MDL\d+$_exc_syn_conductance": "0.4 nS",
        r"^DB\d+_to_MDR\d+$_exc_syn_conductance": "0.4 nS",
        r"^VB\d+_to_MVL\d+$_exc_syn_conductance": "0.6 nS",
        r"^VB\d+_to_MVR\d+$_exc_syn_conductance": "0.6 nS",
        "neuron_to_muscle_exc_syn_vth": "37 mV",
        "neuron_to_muscle_inh_syn_conductance": "0.6 nS",
        "neuron_to_neuron_inh_syn_conductance": "0.2 nS",
        #'DB2_to_MDL11_exc_syn_conductance': '1 nS',
        "AVBR_to_MVL16_exc_syn_conductance": "0 nS",
        "ca_conc_decay_time_muscle": "60.8 ms",
        "ca_conc_rho_muscle": "0.002338919 mol_per_m_per_A_per_s",
    }

    nml_doc = None
    if generate:
        nml_doc = c302.generate(
            reference,
            params,
            cells=cells,
            cells_to_plot=cells_to_plot,
            cells_to_stimulate=cells_to_stimulate,
            conns_to_include=conns_to_include,
            conns_to_exclude=conns_to_exclude,
            conn_polarity_override=conn_polarity_override,
            conn_number_override=conn_number_override,
            muscles_to_include=muscles_to_include,
            duration=duration,
            dt=dt,
            target_directory=target_directory,
            data_reader=data_reader,
            param_overrides=param_overrides,
            verbose=verbose,
        )

        # if config_param_overrides.has_key('input'):
        #    input_list = config_param_overrides['input']

        for stim_input in input_list:
            cell, start, dur, current = stim_input
            c302.add_new_input(nml_doc, cell, start, dur, current, params)

        nml_file = target_directory + "/" + reference + ".net.nml"
        writers.NeuroMLWriter.write(
            nml_doc, nml_file
        )  # Write over network file written above...

        c302.print_("(Re)written network file to: " + nml_file)

    return cells, cells_to_stimulate, params, muscles_to_include, nml_doc


if __name__ == "__main__":
    parameter_set = sys.argv[1] if len(sys.argv) == 2 else "C2"
    data_reader = sys.argv[2] if len(sys.argv) == 3 else c302.FW_DATA_READER

    setup(parameter_set, generate=True, data_reader=data_reader)



import logging
import re

from c302.ConnectomeReader import ConnectionInfo
from c302.ConnectomeReader import analyse_connections
from c302 import print_, MUSCLE_RE

try:
    from owmeta_core.bundle import Bundle
    from owmeta_core.context import Context
    from owmeta.neuron import Neuron
    from owmeta.muscle import BodyWallMuscle
    from owmeta.worm import Worm
except Exception:
    print("owmeta not installed! Cannot run OpenWormReader")
    exit()

############################################################

#   A simple script to read the values in owmeta
#   Originally written by Mark Watts (github.com/mwatts15)

############################################################

LOGGER = logging.getLogger(__name__)


class OpenWormReader(object):
    def __init__(self):
        self.cached = False

    def get_cells_in_model(self, net):
        cell_names = set()
        for n in net.neurons():
            cell_names.add(str(n.name()))

        return cell_names

    def read_data(self, include_nonconnected_cells=False):
        print_("Initialising OpenWormReader")

        try:
            cell_names, pre, post, conns = self._read_connections("neuron")
        except Exception:
            print(
                "\nProblem loading connections via owmeta! The package is installed however. You may need to try running:"
                + "\n\n    owm bundle remote --user add ow 'https://raw.githubusercontent.com/openworm/owmeta-bundles/master/index.json'\n"
            )

            exit()

        if include_nonconnected_cells:
            return cell_names, conns
        else:
            return pre + post, conns

    def read_muscle_data(self):
        cell_names, neurons, muscles, conns = self._read_connections("muscle")
        return neurons, muscles, conns

    def _read_connections(self, termination=None):
        if not self.cached:
            with Bundle("openworm/owmeta-data", version=6) as bnd:
                ctx = bnd(Context)(ident="http://openworm.org/data").stored
                # Extract the network object from the worm object.
                net = ctx(Worm).query().neuron_network()

                syn = net.synapse.expr
                pre = syn.pre_cell
                post = syn.post_cell

                (pre | post).rdf_type(multiple=True)

                (pre | post).name()
                pre()
                post()
                syn.syntype()
                syn.synclass()
                syn.number()
                self.connlist = syn.to_objects()

                self.cell_names = self.get_cells_in_model(net)
            self.cached = True

        if termination == "neuron":
            term_type = set([Neuron.rdf_type])
        elif termination == "muscle":
            term_type = set([BodyWallMuscle.rdf_type])
        else:
            term_type = set([Neuron.rdf_type, BodyWallMuscle.rdf_type])

        conns = []
        pre_cell_names = set()
        post_cell_names = set()
        for conn in self.connlist:
            if Neuron.rdf_type in conn.pre_cell.rdf_type and term_type & set(
                conn.post_cell.rdf_type
            ):
                num = conn.number
                syntype = conn.syntype or ""
                synclass = conn.synclass or ""
                pre_name = conn.pre_cell.name
                post_name = conn.post_cell.name
                if BodyWallMuscle.rdf_type in conn.post_cell.rdf_type:
                    post_name = format_muscle_name(post_name)

                if not synclass:
                    # Hack/guess
                    if syntype and syntype.lower() == "gapjunction":
                        synclass = "Generic_GJ"
                    else:
                        if pre_name.startswith("DD") or pre_name.startswith("VD"):
                            synclass = "GABA"
                        synclass = "Acetylcholine"
                conns.append(
                    ConnectionInfo(pre_name, post_name, num, syntype, synclass)
                )

                pre_cell_names.add(pre_name)
                post_cell_names.add(post_name)

        print_(
            "Total cells %i (%i with connections)"
            % (
                len(self.cell_names | pre_cell_names | post_cell_names),
                len(pre_cell_names | post_cell_names),
            )
        )
        print_("Total connections found %i " % len(conns))

        return list(self.cell_names), pre_cell_names, post_cell_names, conns


def format_muscle_name(muscle_name):
    md = MUSCLE_RE.fullmatch(muscle_name)
    if md:
        return muscle_name
    else:
        md = re.fullmatch(r"([VD][LR])(\d+)", muscle_name)
        if md:
            return "M{0}{1:02d}".format(md.group(1), int(md.group(2)))
        else:
            LOGGER.debug("Unrecognized muscle name format in %s", muscle_name)
            return muscle_name


READER = OpenWormReader()

read_data = READER.read_data
read_muscle_data = READER.read_muscle_data

if __name__ == "__main__":
    cells, neuron_conns = read_data(include_nonconnected_cells=True)
    neurons2muscles, muscles, muscle_conns = read_muscle_data()

    analyse_connections(cells, neuron_conns, neurons2muscles, muscles, muscle_conns)

    exit()

    conn_map_OWR = {}
    for c in neuron_conns:
        conn_map_OWR[c.short().lower()] = c

    from c302.UpdatedSpreadsheetDataReader import read_data as read_data_usr

    # from c302.SpreadsheetDataReader import read_data as read_data_usr

    cells2, conns2 = read_data_usr(include_nonconnected_cells=True)

    print_(
        "%i cells found using UpdatedSpreadsheetDataReader2: %s..."
        % (len(cells2), sorted(cells2)[0:3])
    )
    print_(
        "Found %s connections using UpdatedSpreadsheetDataReader2, First few: "
        % (len(conns2),)
    )
    for c in sorted(conns2)[: min(len(conns2), 5)]:
        print_("  %s" % c)

    conn_map_USR = {}
    for c2 in conns2:
        conn_map_USR[c2.short().lower()] = c2

    maxn = 3

    refs_OWR = list(conn_map_OWR.keys())

    matching = 0

    for i in range(min(maxn, len(refs_OWR))):
        ref = refs_OWR[i]
        if ref in conn_map_USR:
            if conn_map_OWR[ref].number != conn_map_USR[ref].number:
                print_("Mismatch: %s != %s" % (conn_map_OWR[ref], conn_map_USR[ref]))
            else:
                matching += 1
        else:
            print_(
                "Missing from UpdatedSpreadsheetDataReader: %s" % (conn_map_OWR[ref])
            )

    print_("Number matching: %i" % matching)

    matching = 0

    refs_USR = list(conn_map_USR.keys())

    for i in range(min(maxn, len(refs_USR))):
        # print("\n-----  Connection in USR: %s"%refs[i])
        # print cm2[refs[i]]
        ref = refs_USR[i]
        if ref in conn_map_OWR:
            if conn_map_OWR[ref].number != conn_map_USR[ref].number:
                print_("Mismatch: %s != %s" % (conn_map_OWR[ref], conn_map_USR[ref]))
            else:
                matching += 1
        else:
            print_("* Missing from OpenWormReader: %s" % conn_map_USR[ref])

    print_("Number matching: %i" % matching)



from c302.ConnectomeReader import ConnectionInfo
from c302.ConnectomeReader import analyse_connections
from openpyxl import load_workbook

import os
from c302 import print_

spreadsheet_location = os.path.dirname(os.path.abspath(__file__)) + "/data/"
spreadsheet_name = "NeuronConnect.xlsx"  # has old name...
spreadsheet_name = "NeuronConnectFormatted.xlsx"

READER_DESCRIPTION = (
    """Data extracted from %s for neuronal connectivity""" % spreadsheet_name
)

NMJ_ENDPOINT = "NMJ"


def read_data(include_nonconnected_cells=False, neuron_connect=True):
    if neuron_connect:
        conns = []
        cells = []
        filename = "%s%s" % (spreadsheet_location, spreadsheet_name)
        wb = load_workbook(filename)
        sheet = wb.worksheets[0]
        print_("Opened the Excel file: " + filename)

        for row in sheet.iter_rows(
            min_row=2, values_only=True
        ):  # Assuming data starts from the second row
            pre = str(row[0])
            post = str(row[1])

            if not post == NMJ_ENDPOINT:
                syntype = str(row[2])
                num = int(row[3])
                synclass = "Generic_GJ" if "EJ" in syntype else "Generic_CS"

                conns.append(ConnectionInfo(pre, post, num, syntype, synclass))
                if pre not in cells:
                    cells.append(pre)
                if post not in cells:
                    cells.append(post)

        return cells, conns


def read_muscle_data():
    conns = []
    neurons = []
    muscles = []

    filename = "%s%s" % (spreadsheet_location, spreadsheet_name)
    wb = load_workbook(filename)
    sheet = wb.worksheets[0]

    print_("Opened Excel file: " + filename)

    for row in sheet.iter_rows(
        min_row=2, values_only=True
    ):  # Assuming data starts from the second row
        pre = str(row[0])
        post = str(row[1])

        if post == NMJ_ENDPOINT:
            syntype = str(row[2])
            num = int(row[3])
            synclass = "Generic_GJ" if "EJ" in syntype else "Generic_CS"

            conns.append(ConnectionInfo(pre, post, num, syntype, synclass))
            if pre not in neurons:
                neurons.append(pre)
            if post not in muscles:
                muscles.append(post)

    return neurons, muscles, conns


def main():
    cells, neuron_conns = read_data(include_nonconnected_cells=True)
    neurons2muscles, muscles, muscle_conns = read_muscle_data()

    analyse_connections(cells, neuron_conns, neurons2muscles, muscles, muscle_conns)


if __name__ == "__main__":
    main()
